package javax.swing;

import sun.awt.AWTAccessor;
import sun.awt.SunToolkit;
import java.awt.Toolkit;
import java.awt.Component;
import java.awt.Dimension;
import sun.java2d.SunGraphics2D;
import java.awt.Color;
import java.awt.Window;
import com.sun.awt.AWTUtilities;
import java.awt.Container;
import java.beans.PropertyVetoException;
import java.awt.Point;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.io.Serializable;

public class DefaultDesktopManager implements DesktopManager, Serializable
{
    static final String HAS_BEEN_ICONIFIED_PROPERTY = "wasIconOnce";
    static final int DEFAULT_DRAG_MODE = 0;
    static final int OUTLINE_DRAG_MODE = 1;
    static final int FASTER_DRAG_MODE = 2;
    int dragMode;
    private transient Rectangle currentBounds;
    private transient Graphics desktopGraphics;
    private transient Rectangle desktopBounds;
    private transient Rectangle[] floatingItems;
    private transient boolean didDrag;
    private transient Point currentLoc;
    
    public DefaultDesktopManager() {
        this.dragMode = 0;
        this.currentBounds = null;
        this.desktopGraphics = null;
        this.desktopBounds = null;
        this.floatingItems = new Rectangle[0];
        this.currentLoc = null;
    }
    
    @Override
    public void openFrame(JInternalFrame comp) {
        if (comp.getDesktopIcon().getParent() != null) {
            comp.getDesktopIcon().getParent().add(comp);
            this.removeIconFor(comp);
        }
    }
    
    @Override
    public void closeFrame(JInternalFrame comp) {
        JDesktopPane desktopPane = comp.getDesktopPane();
        if (desktopPane == null) {
            return;
        }
        int selected = comp.isSelected();
        Container parent = comp.getParent();
        JInternalFrame nextFrame = null;
        if (!(selected == 0)) {
            nextFrame = desktopPane.getNextFrame(comp);
            try {
                comp.setSelected(false);
            }
            catch (PropertyVetoException ex) {}
        }
        if (parent != null) {
            parent.remove(comp);
            parent.repaint(comp.getX(), comp.getY(), comp.getWidth(), comp.getHeight());
        }
        this.removeIconFor(comp);
        if (comp.getNormalBounds() != null) {
            comp.setNormalBounds(null);
        }
        if (this.wasIcon(comp)) {
            this.setWasIcon(comp, null);
        }
        if (nextFrame != null) {
            try {
                nextFrame.setSelected(true);
            }
            catch (PropertyVetoException ex2) {}
        }
        else if (selected && desktopPane.getComponentCount() == 0) {
            desktopPane.requestFocus();
        }
        return;
    }
    
    @Override
    public void maximizeFrame(JInternalFrame internalFrame) {
        if (internalFrame.isIcon()) {
            try {
                internalFrame.setIcon(false);
            }
            catch (PropertyVetoException ex) {}
        }
        else {
            internalFrame.setNormalBounds(internalFrame.getBounds());
            Rectangle bounds = internalFrame.getParent().getBounds();
            this.setBoundsForFrame(internalFrame, 0, 0, bounds.width, bounds.height);
        }
        try {
            internalFrame.setSelected(true);
        }
        catch (PropertyVetoException ex2) {}
    }
    
    @Override
    public void minimizeFrame(JInternalFrame internalFrame) {
        if (internalFrame.isIcon()) {
            this.iconifyFrame(internalFrame);
            return;
        }
        if (internalFrame.getNormalBounds() != null) {
            Rectangle normalBounds = internalFrame.getNormalBounds();
            internalFrame.setNormalBounds(null);
            try {
                internalFrame.setSelected(true);
            }
            catch (PropertyVetoException ex) {}
            this.setBoundsForFrame(internalFrame, normalBounds.x, normalBounds.y, normalBounds.width, normalBounds.height);
        }
        return;
    }
    
    @Override
    public void iconifyFrame(JInternalFrame comp) {
        Container parent = comp.getParent();
        JDesktopPane desktopPane = comp.getDesktopPane();
        int selected = comp.isSelected();
        JInternalFrame.JDesktopIcon desktopIcon = comp.getDesktopIcon();
        if (!this.wasIcon(comp)) {
            Rectangle boundsForIcon = this.getBoundsForIconOf(comp);
            desktopIcon.setBounds(boundsForIcon.x, boundsForIcon.y, boundsForIcon.width, boundsForIcon.height);
            desktopIcon.revalidate();
            this.setWasIcon(comp, Boolean.TRUE);
        }
        if (parent == null || desktopPane == null) {
            return;
        }
        if ((parent instanceof JLayeredPane)) {
            JLayeredPane layeredPane = (JLayeredPane)parent;
            JLayeredPane.putLayer(desktopIcon, JLayeredPane.getLayer(comp));
        }
        if (!comp.isMaximum()) {
            comp.setNormalBounds(comp.getBounds());
        }
        desktopPane.setComponentOrderCheckingEnabled(false);
        parent.remove(comp);
        parent.add(desktopIcon);
        desktopPane.setComponentOrderCheckingEnabled(true);
        parent.repaint(comp.getX(), comp.getY(), comp.getWidth(), comp.getHeight());
        if (selected && desktopPane.selectFrame(true) == null) {
            comp.restoreSubcomponentFocus();
        }
        return;
    }
    
    @Override
    public void deiconifyFrame(JInternalFrame comp) {
        Container parent = comp.getDesktopIcon().getParent();
        JDesktopPane desktopPane = comp.getDesktopPane();
        if (parent != null && desktopPane != null) {
            parent.add(comp);
            if (comp.isMaximum()) {
                Rectangle bounds = parent.getBounds();
                if (comp.getWidth() != bounds.width || comp.getHeight() != bounds.height) {
                    this.setBoundsForFrame(comp, 0, 0, bounds.width, bounds.height);
                }
            }
            this.removeIconFor(comp);
            if (comp.isSelected()) {
                comp.moveToFront();
                comp.restoreSubcomponentFocus();
            }
            else {
                try {
                    comp.setSelected(true);
                }
                catch (PropertyVetoException ex) {}
            }
        }
    }
    
    @Override
    public void activateFrame(JInternalFrame internalFrame) {
        Container parent = internalFrame.getParent();
        JDesktopPane desktopPane = internalFrame.getDesktopPane();
        JInternalFrame internalFrame2 = (desktopPane == null) ? null : desktopPane.getSelectedFrame();
        if (parent == null && internalFrame.getDesktopIcon().getParent() == null) {
            return;
        }
        if (internalFrame2 == null) {
            if (desktopPane != null) {
                desktopPane.setSelectedFrame(internalFrame);
            }
        }
        else if (internalFrame2 != internalFrame) {
            if (internalFrame2.isSelected()) {
                try {
                    internalFrame2.setSelected(false);
                }
                catch (PropertyVetoException ex) {}
            }
            if (desktopPane != null) {
                desktopPane.setSelectedFrame(internalFrame);
            }
        }
        internalFrame.moveToFront();
        return;
    }
    
    @Override
    public void deactivateFrame(JInternalFrame internalFrame) {
        JDesktopPane desktopPane = internalFrame.getDesktopPane();
        if (((desktopPane == null) ? null : desktopPane.getSelectedFrame()) == internalFrame) {
            desktopPane.setSelectedFrame(null);
        }
    }
    
    @Override
    public void beginDraggingFrame(JComponent component) {
        this.setupDragMode(component);
        if (this.dragMode == 2) {
            Container parent = component.getParent();
            this.floatingItems = this.findFloatingItems(component);
            this.currentBounds = component.getBounds();
            if ((parent instanceof JComponent)) {
                this.desktopBounds = ((JComponent)parent).getVisibleRect();
            }
            else {
                this.desktopBounds = parent.getBounds();
                Rectangle desktopBounds = this.desktopBounds;
                Rectangle desktopBounds2 = this.desktopBounds;
                int n = 0;
                desktopBounds2.y = n;
                desktopBounds.x = n;
            }
            this.desktopGraphics = JComponent.safelyGetGraphics(parent);
            ((JInternalFrame)component).isDragging = true;
            this.didDrag = false;
        }
    }
    
    private void setupDragMode(JComponent component) {
        JDesktopPane desktopPane = this.getDesktopPane(component);
        Container parent = component.getParent();
        this.dragMode = 0;
        if (desktopPane != null) {
            String s = (String)desktopPane.getClientProperty("JDesktopPane.dragMode");
            Window windowAncestor = SwingUtilities.getWindowAncestor(component);
            if (windowAncestor != null && !AWTUtilities.isWindowOpaque(windowAncestor)) {
                this.dragMode = 0;
            }
            else if (s != null && s.equals("outline")) {
                this.dragMode = 1;
            }
            else if (s != null && s.equals("faster") && component instanceof JInternalFrame && ((JComponent)component).isOpaque() && (parent == null || parent.isOpaque())) {
                this.dragMode = 2;
            }
            else if (desktopPane.getDragMode() == 1) {
                this.dragMode = 1;
            }
            else if (desktopPane.getDragMode() == 0 && component instanceof JInternalFrame && ((JComponent)component).isOpaque()) {
                this.dragMode = 2;
            }
            else {
                this.dragMode = 0;
            }
        }
    }
    
    @Override
    public void dragFrame(JComponent component, int n, int n2) {
        if (this.dragMode == 1) {
            JDesktopPane desktopPane = this.getDesktopPane(component);
            if (desktopPane != null) {
                Graphics safelyGetGraphics = JComponent.safelyGetGraphics(desktopPane);
                safelyGetGraphics.setXORMode(Color.white);
                if (this.currentLoc != null) {
                    safelyGetGraphics.drawRect(this.currentLoc.x, this.currentLoc.y, component.getWidth() - 1, component.getHeight() - 1);
                }
                safelyGetGraphics.drawRect(n, n2, component.getWidth() - 1, component.getHeight() - 1);
                if (!((SunGraphics2D)safelyGetGraphics).getSurfaceData().isSurfaceLost()) {
                    this.currentLoc = new Point(n, n2);
                }
                safelyGetGraphics.dispose();
            }
        }
        else if (this.dragMode == 2) {
            this.dragFrameFaster(component, n, n2);
        }
        else {
            this.setBoundsForFrame(component, n, n2, component.getWidth(), component.getHeight());
        }
    }
    
    @Override
    public void endDraggingFrame(JComponent component) {
        if (this.dragMode == 1 && this.currentLoc != null) {
            this.setBoundsForFrame(component, this.currentLoc.x, this.currentLoc.y, component.getWidth(), component.getHeight());
            this.currentLoc = null;
        }
        else if (this.dragMode == 2) {
            this.currentBounds = null;
            if (this.desktopGraphics != null) {
                this.desktopGraphics.dispose();
                this.desktopGraphics = null;
            }
            this.desktopBounds = null;
            ((JInternalFrame)component).isDragging = false;
        }
    }
    
    @Override
    public void beginResizingFrame(JComponent component, int n) {
        this.setupDragMode(component);
    }
    
    @Override
    public void resizeFrame(JComponent component, int n, int n2, int width, int height) {
        if (this.dragMode == 0 || this.dragMode == 2) {
            this.setBoundsForFrame(component, n, n2, width, height);
        }
        else {
            JDesktopPane desktopPane = this.getDesktopPane(component);
            if (desktopPane != null) {
                Graphics safelyGetGraphics = JComponent.safelyGetGraphics(desktopPane);
                safelyGetGraphics.setXORMode(Color.white);
                if (this.currentBounds != null) {
                    safelyGetGraphics.drawRect(this.currentBounds.x, this.currentBounds.y, this.currentBounds.width - 1, this.currentBounds.height - 1);
                }
                safelyGetGraphics.drawRect(n, n2, width - 1, height - 1);
                if (!((SunGraphics2D)safelyGetGraphics).getSurfaceData().isSurfaceLost()) {
                    this.currentBounds = new Rectangle(n, n2, width, height);
                }
                safelyGetGraphics.setPaintMode();
                safelyGetGraphics.dispose();
            }
        }
    }
    
    @Override
    public void endResizingFrame(JComponent component) {
        if (this.dragMode == 1 && this.currentBounds != null) {
            this.setBoundsForFrame(component, this.currentBounds.x, this.currentBounds.y, this.currentBounds.width, this.currentBounds.height);
            this.currentBounds = null;
        }
    }
    
    @Override
    public void setBoundsForFrame(JComponent component, int x, int y, int width, int height) {
        component.setBounds(x, y, width, height);
        component.revalidate();
    }
    
    protected void removeIconFor(JInternalFrame internalFrame) {
        JInternalFrame.JDesktopIcon desktopIcon = internalFrame.getDesktopIcon();
        Container parent = desktopIcon.getParent();
        if (parent != null) {
            parent.remove(desktopIcon);
            parent.repaint(desktopIcon.getX(), desktopIcon.getY(), desktopIcon.getWidth(), desktopIcon.getHeight());
        }
    }
    
    protected Rectangle getBoundsForIconOf(JInternalFrame internalFrame) {
        JInternalFrame.JDesktopIcon desktopIcon = internalFrame.getDesktopIcon();
        Dimension preferredSize = desktopIcon.getPreferredSize();
        Container container = internalFrame.getParent();
        if (container == null) {
            container = internalFrame.getDesktopIcon().getParent();
        }
        if (container == null) {
            return new Rectangle(0, 0, preferredSize.width, preferredSize.height);
        }
        Rectangle bounds = container.getBounds();
        Component[] components = container.getComponents();
        Object o = null;
        JInternalFrame.JDesktopIcon desktopIcon2 = null;
        int x = 0;
        int y = bounds.height - preferredSize.height;
        int width = preferredSize.width;
        int height = preferredSize.height;
        int i = 0;
        while (i == 0) {
            o = new Rectangle(x, y, width, height);
            i = 1;
            for (int j = 0; j < components.length; ++j) {
                if ((components[j] instanceof JInternalFrame)) {
                    desktopIcon2 = ((JInternalFrame)components[j]).getDesktopIcon();
                }
                else if ((components[j] instanceof JInternalFrame.JDesktopIcon)) {
                    desktopIcon2 = (JInternalFrame.JDesktopIcon)components[j];
                }
                else {
                    continue;
                }
                if (!desktopIcon2.equals(desktopIcon) && o.intersects(desktopIcon2.getBounds())) {
                    i = 0;
                    break;
                }
            }
            if (desktopIcon2 == null) {
                return o;
            }
            x += desktopIcon2.getBounds().width;
            if (x + width <= bounds.width) {
                continue;
            }
            x = 0;
            y -= height;
        }
        return o;
    }
    
    protected void setPreviousBounds(JInternalFrame internalFrame, Rectangle normalBounds) {
        internalFrame.setNormalBounds(normalBounds);
    }
    
    protected Rectangle getPreviousBounds(JInternalFrame internalFrame) {
        return internalFrame.getNormalBounds();
    }
    
    protected void setWasIcon(JInternalFrame internalFrame, Boolean b) {
        if (b != null) {
            internalFrame.putClientProperty("wasIconOnce", b);
        }
    }
    
    protected boolean wasIcon(JInternalFrame internalFrame) {
        return internalFrame.getClientProperty("wasIconOnce") == Boolean.TRUE;
    }
    
    JDesktopPane getDesktopPane(JComponent component) {
        JDesktopPane desktopPane = null;
        Container container = component.getParent();
        while (desktopPane == null) {
            if ((container instanceof JDesktopPane)) {
                desktopPane = (JDesktopPane)container;
            }
            else {
                if (container == null) {
                    break;
                }
                container = container.getParent();
            }
        }
        return desktopPane;
    }
    
    private void dragFrameFaster(JComponent component, int x, int y) {
        Rectangle obj = new Rectangle(this.currentBounds.x, this.currentBounds.y, this.currentBounds.width, this.currentBounds.height);
        this.currentBounds.x = x;
        this.currentBounds.y = y;
        if (this.didDrag) {
            this.emergencyCleanup(component);
        }
        else {
            this.didDrag = true;
            ((JInternalFrame)component).danger = false;
        }
        int floaterCollision = this.isFloaterCollision(obj, this.currentBounds);
        JComponent component2 = (JComponent)component.getParent();
        Rectangle intersection = obj.intersection(this.desktopBounds);
        RepaintManager currentManager = RepaintManager.currentManager(component);
        currentManager.beginPaint();
        try {
            if (floaterCollision == 0) {
                currentManager.copyArea(component2, this.desktopGraphics, intersection.x, intersection.y, intersection.width, intersection.height, x - obj.x, y - obj.y, true);
            }
            component.setBounds(this.currentBounds);
            if (floaterCollision == 0) {
                Rectangle currentBounds = this.currentBounds;
                currentManager.notifyRepaintPerformed(component2, currentBounds.x, currentBounds.y, currentBounds.width, currentBounds.height);
            }
            if (!(floaterCollision == 0)) {
                ((JInternalFrame)component).isDragging = false;
                component2.paintImmediately(this.currentBounds);
                ((JInternalFrame)component).isDragging = true;
            }
            currentManager.markCompletelyClean(component2);
            currentManager.markCompletelyClean(component);
            Rectangle[] computeDifference;
            if (obj.intersects(this.currentBounds)) {
                computeDifference = SwingUtilities.computeDifference(obj, this.currentBounds);
            }
            else {
                computeDifference = new Rectangle[] { obj };
            }
            for (int i = 0; i < computeDifference.length; ++i) {
                component2.paintImmediately(computeDifference[i]);
                Rectangle rectangle = computeDifference[i];
                currentManager.notifyRepaintPerformed(component2, rectangle.x, rectangle.y, rectangle.width, rectangle.height);
            }
            if (!intersection.equals(obj)) {
                Rectangle[] computeDifference2 = SwingUtilities.computeDifference(obj, this.desktopBounds);
                for (int j = 0; j < computeDifference2.length; ++j) {
                    Rectangle rectangle2 = computeDifference2[j];
                    rectangle2.x += x - obj.x;
                    Rectangle rectangle3 = computeDifference2[j];
                    rectangle3.y += y - obj.y;
                    ((JInternalFrame)component).isDragging = false;
                    component2.paintImmediately(computeDifference2[j]);
                    ((JInternalFrame)component).isDragging = true;
                    Rectangle rectangle4 = computeDifference2[j];
                    currentManager.notifyRepaintPerformed(component2, rectangle4.x, rectangle4.y, rectangle4.width, rectangle4.height);
                }
            }
        }
        finally {
            currentManager.endPaint();
        }
        Window windowAncestor = SwingUtilities.getWindowAncestor(component);
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if (!windowAncestor.isOpaque() && defaultToolkit instanceof SunToolkit && ((SunToolkit)defaultToolkit).needUpdateWindow()) {
            AWTAccessor.getWindowAccessor().updateWindow(windowAncestor);
        }
    }
    
    private boolean isFloaterCollision(Rectangle rectangle, Rectangle rectangle2) {
        if (this.floatingItems.length == 0) {
            return false;
        }
        int i = 0;
        while (i < this.floatingItems.length) {
            if (rectangle.intersects(this.floatingItems[i])) {
                return true;
            }
            if (rectangle2.intersects(this.floatingItems[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private Rectangle[] findFloatingItems(JComponent component) {
        Component[] components;
        int n;
        for (components = component.getParent().getComponents(), n = 0; n < components.length && components[n] != component; ++n) {}
        Rectangle[] array = new Rectangle[n];
        for (int i = 0; i < array.length; ++i) {
            array[i] = components[i].getBounds();
        }
        return array;
    }
    
    private void emergencyCleanup(final JComponent component) {
        if (((JInternalFrame)component).danger) {
            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {
                    ((JInternalFrame)component).isDragging = false;
                    component.paintImmediately(0, 0, component.getWidth(), component.getHeight());
                    ((JInternalFrame)component).isDragging = true;
                }
            });
            ((JInternalFrame)component).danger = false;
        }
    }
}
